@@ -39,14 +39,25 @@ check_path_is_directory <- function(path) {
     ui_stop("Directory {ui_path(path)} does not exist.")
   }
 
+  if (is_link(path)) {
+    path <- link_path(path)
+  }
+
   if (!is_dir(path)) {
     ui_stop("{ui_path(path)} is not a directory.")
   }
 }
 
+count_directory_files <- function(x) {
+  length(dir_ls(x))
+}
+
+directory_has_files <- function(x) {
+  count_directory_files(x) >= 1
+}
+
 check_directory_is_empty <- function(x) {
-  files <- dir_ls(x)
-  if (length(files) > 0) {
+  if (directory_has_files(x)) {
     ui_stop("{ui_path(x)} exists and is not an empty directory.")
   }
   invisible(x)
