@@ -1,5 +1,3 @@
-context("projects")
-
 test_that("proj_set() errors on non-existent path", {
   expect_usethis_error(
     proj_set("abcedefgihklmnopqrstuv"),
@@ -8,9 +6,7 @@ test_that("proj_set() errors on non-existent path", {
 })
 
 test_that("proj_set() errors if no criteria are fulfilled", {
-  tmpdir <- file_temp(pattern = "i-am-not-a-project")
-  on.exit(dir_delete(tmpdir))
-  dir_create(tmpdir)
+  tmpdir <- withr::local_tempdir(pattern = "i-am-not-a-project")
   expect_usethis_error(
     proj_set(tmpdir),
     "does not appear to be inside a project or package"
@@ -18,41 +14,33 @@ test_that("proj_set() errors if no criteria are fulfilled", {
 })
 
 test_that("proj_set() can be forced, even if no criteria are fulfilled", {
-  tmpdir <- file_temp(pattern = "i-am-not-a-project")
+  tmpdir <- withr::local_tempdir(pattern = "i-am-not-a-project")
 
-  on.exit(dir_delete(tmpdir))
-  dir_create(tmpdir)
   expect_error_free(old <- proj_set(tmpdir, force = TRUE))
-  on.exit(proj_set(old), add = TRUE)
+  withr::defer(proj_set(old))
   expect_identical(proj_get(), proj_path_prep(tmpdir))
-
-  proj_set_(tempdir())
-  expect_usethis_error(
-    proj_set(proj_get()),
-    "does not appear to be inside a project or package"
-  )
 })
 
 test_that("is_package() detects package-hood", {
-  scoped_temporary_package()
+  create_local_package()
   expect_true(is_package())
 
-  scoped_temporary_project()
+  create_local_project()
   expect_false(is_package())
 })
 
 test_that("check_is_package() errors for non-package", {
-  scoped_temporary_project()
+  create_local_project()
   expect_usethis_error(check_is_package(), "not an R package")
 })
 
 test_that("check_is_package() can reveal who's asking", {
-  scoped_temporary_project()
+  create_local_project()
   expect_usethis_error(check_is_package("foo"), "foo")
 })
 
 test_that("proj_path() appends to the project path", {
-  scoped_temporary_project()
+  create_local_project()
   expect_equal(
     proj_path("a", "b", "c"),
     path(proj_get(), "a/b/c")
@@ -61,40 +49,37 @@ test_that("proj_path() appends to the project path", {
 })
 
 test_that("proj_rel_path() returns path part below the project", {
-  scoped_temporary_project()
+  create_local_project()
   expect_equal(proj_rel_path(proj_path("a/b/c")), "a/b/c")
 })
 
 test_that("proj_rel_path() returns path 'as is' if not in project", {
-  scoped_temporary_project()
+  create_local_project()
   expect_identical(proj_rel_path(path_temp()), path_temp())
 })
 
 test_that("proj_set() enforces proj path preparation policy", {
-  ## specifically: check that proj_get() returns realized path
-  t <- dir_create(file_temp())
+  # specifically: check that proj_get() returns realized path
+  t <- withr::local_tempdir("proj-set-path-prep")
 
-  ## a/b/d and a/b2/d identify same directory
+  # a/b/d and a/b2/d identify same directory
   a <- path_real(dir_create(path(t, "a")))
   b <- dir_create(path(a, "b"))
   b2 <- link_create(b, path(a, "b2"))
   d <- dir_create(path(b, "d"))
 
-  ## input path includes symbolic link
+  # input path includes symbolic link
   path_with_symlinks <- path(b2, "d")
-  expect_equal(path_rel(path_with_symlinks, a), "b2/d")
+  expect_equal(path_rel(path_with_symlinks, a), path("b2/d"))
 
-  ## force = TRUE
-  old <- proj_set(path_with_symlinks, force = TRUE)
-  on.exit(proj_set(old))
-  expect_equal(path_rel(proj_get(), a), "b/d")
+  # force = TRUE
+  local_project(path_with_symlinks, force = TRUE)
+  expect_equal(path_rel(proj_get(), a), path("b/d"))
 
-  ## force = FALSE
+  # force = FALSE
   file_create(path(b, "d", ".here"))
   proj_set(path_with_symlinks, force = FALSE)
-  expect_equal(path_rel(proj_get(), a), "b/d")
-
-  dir_delete(t)
+  expect_equal(path_rel(proj_get(), a), path("b/d"))
 })
 
 test_that("proj_path_prep() passes NULL through", {
@@ -102,7 +87,7 @@ test_that("proj_path_prep() passes NULL through", {
 })
 
 test_that("is_in_proj() detects whether files are (or would be) in project", {
-  scoped_temporary_package()
+  create_local_package()
 
   ## file does not exist but would be in project if created
   expect_true(is_in_proj(proj_path("fiction")))
@@ -117,8 +102,17 @@ test_that("is_in_proj() detects whether files are (or would be) in project", {
   expect_false(is_in_proj(path_temp()))
 })
 
+test_that("is_in_proj() does not activate a project", {
+  pkg <- create_local_package()
+  path <- proj_path("DESCRIPTION")
+  expect_true(is_in_proj(path))
+  local_project(NULL)
+  expect_false(is_in_proj(path))
+  expect_false(proj_active())
+})
+
 test_that("proj_sitrep() reports current working/project state", {
-  pkg <- scoped_temporary_package()
+  pkg <- create_local_package()
   x <- proj_sitrep()
   expect_s3_class(x, "sitrep")
   expect_false(is.null(x[["working_directory"]]))
@@ -130,10 +124,11 @@ test_that("proj_sitrep() reports current working/project state", {
 
 test_that("with_project() runs code in temp proj, restores (lack of) proj", {
   old_project <- proj_get_()
-  on.exit(proj_set_(old_project))
+  withr::defer(proj_set_(old_project))
 
   temp_proj <- create_project(
-    file_temp(pattern = "TEMPPROJ"), rstudio = FALSE, open = FALSE
+    file_temp(pattern = "TEMPPROJ"),
+    rstudio = FALSE, open = FALSE
   )
 
   proj_set_(NULL)
@@ -147,13 +142,15 @@ test_that("with_project() runs code in temp proj, restores (lack of) proj", {
 
 test_that("with_project() runs code in temp proj, restores original proj", {
   old_project <- proj_get_()
-  on.exit(proj_set_(old_project))
+  withr::defer(proj_set_(old_project))
 
   host <- create_project(
-    file_temp(pattern = "host"), rstudio = FALSE, open = FALSE
+    file_temp(pattern = "host"),
+    rstudio = FALSE, open = FALSE
   )
   guest <- create_project(
-    file_temp(pattern = "guest"), rstudio = FALSE, open = FALSE
+    file_temp(pattern = "guest"),
+    rstudio = FALSE, open = FALSE
   )
 
   proj_set(host)
@@ -167,10 +164,11 @@ test_that("with_project() runs code in temp proj, restores original proj", {
 
 test_that("with_project() works when temp proj == original proj", {
   old_project <- proj_get_()
-  on.exit(proj_set_(old_project))
+  withr::defer(proj_set_(old_project))
 
   host <- create_project(
-    file_temp(pattern = "host"), rstudio = FALSE, open = FALSE
+    file_temp(pattern = "host"),
+    rstudio = FALSE, open = FALSE
   )
 
   proj_set(host)
@@ -184,9 +182,9 @@ test_that("with_project() works when temp proj == original proj", {
 
 test_that("local_project() activates proj til scope ends", {
   old_project <- proj_get_()
-  on.exit(proj_set_(old_project))
+  withr::defer(proj_set_(old_project))
 
-  new_proj <- file_temp(pattern = "aaa")
+  new_proj <- file_temp(pattern = "localprojtest")
   create_project(new_proj, rstudio = FALSE, open = FALSE)
   proj_set_(NULL)
 
@@ -202,3 +200,25 @@ test_that("local_project() activates proj til scope ends", {
   )
   expect_null(proj_get_())
 })
+
+# https://github.com/r-lib/usethis/issues/954
+test_that("proj_activate() works with relative path when RStudio is not detected", {
+  sandbox <- path_real(dir_create(file_temp("sandbox")))
+  withr::defer(dir_delete(sandbox))
+  orig_proj <- proj_get_()
+  withr::defer(proj_set(orig_proj, force = TRUE))
+  withr::local_dir(sandbox)
+
+  rel_path_proj <- path_file(file_temp(pattern = "mno"))
+  out_path <- create_project(rel_path_proj, rstudio = FALSE, open = FALSE)
+  with_mock(
+    # make sure we act as if not in RStudio
+    rstudio_available = function(...) FALSE,
+    expect_error_free(
+      result <- proj_activate(rel_path_proj)
+    )
+  )
+  expect_true(result)
+  expect_equal(path_wd(), out_path)
+  expect_equal(proj_get(), out_path)
+})
